@@ -15,6 +15,7 @@
 package prometheus
 
 import (
+	"context"
 	"fmt"
 	"reflect"
 	"sort"
@@ -57,7 +58,7 @@ func (c *Operator) createOrUpdateRuleConfigMaps(p *monitoringv1.Prometheus) ([]s
 		return nil, err
 	}
 
-	currentConfigMapList, err := cClient.List(prometheusRulesConfigMapSelector(p.Name))
+	currentConfigMapList, err := cClient.List(context.TODO(), prometheusRulesConfigMapSelector(p.Name))
 	if err != nil {
 		return nil, err
 	}
@@ -101,7 +102,7 @@ func (c *Operator) createOrUpdateRuleConfigMaps(p *monitoringv1.Prometheus) ([]s
 			"prometheus", p.Name,
 		)
 		for _, cm := range newConfigMaps {
-			_, err = cClient.Create(&cm)
+			_, err = cClient.Create(context.TODO(), &cm, metav1.CreateOptions{})
 			if err != nil {
 				return nil, errors.Wrapf(err, "failed to create ConfigMap '%v'", cm.Name)
 			}
@@ -112,7 +113,7 @@ func (c *Operator) createOrUpdateRuleConfigMaps(p *monitoringv1.Prometheus) ([]s
 	// Simply deleting old ConfigMaps and creating new ones for now. Could be
 	// replaced by logic that only deletes obsolete ConfigMaps in the future.
 	for _, cm := range currentConfigMaps {
-		err := cClient.Delete(cm.Name, &metav1.DeleteOptions{})
+		err := cClient.Delete(context.TODO(), cm.Name, metav1.DeleteOptions{})
 		if err != nil {
 			return nil, errors.Wrapf(err, "failed to delete current ConfigMap '%v'", cm.Name)
 		}
@@ -124,7 +125,7 @@ func (c *Operator) createOrUpdateRuleConfigMaps(p *monitoringv1.Prometheus) ([]s
 		"prometheus", p.Name,
 	)
 	for _, cm := range newConfigMaps {
-		_, err = cClient.Create(&cm)
+		_, err = cClient.Create(context.TODO(), &cm, metav1.CreateOptions{})
 		if err != nil {
 			return nil, errors.Wrapf(err, "failed to create new ConfigMap '%v'", cm.Name)
 		}
