@@ -15,6 +15,7 @@
 package e2e
 
 import (
+	"context"
 	"flag"
 	"log"
 	"os"
@@ -31,6 +32,24 @@ var (
 	opImage   *string
 )
 
+func skipPrometheusTests(t *testing.T) {
+	if os.Getenv("EXCLUDE_PROMETHEUS_TESTS") != "" {
+		t.Skip("Skipping Prometheus tests")
+	}
+}
+
+func skipAlertmanagerTests(t *testing.T) {
+	if os.Getenv("EXCLUDE_ALERTMANAGER_TESTS") != "" {
+		t.Skip("Skipping Alertmanager tests")
+	}
+}
+
+func skipThanosRulerTests(t *testing.T) {
+	if os.Getenv("EXCLUDE_THANOSRULER_TESTS") != "" {
+		t.Skip("Skipping ThanosRuler tests")
+	}
+}
+
 func TestMain(m *testing.M) {
 	kubeconfig := flag.String(
 		"kubeconfig",
@@ -67,7 +86,7 @@ func TestAllNS(t *testing.T) {
 
 	ns := ctx.CreateNamespace(t, framework.KubeClient)
 
-	finalizers, err := framework.CreatePrometheusOperator(ns, *opImage, nil, nil, true)
+	finalizers, err := framework.CreatePrometheusOperator(ns, *opImage, nil, nil, nil, nil, true)
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -76,19 +95,25 @@ func TestAllNS(t *testing.T) {
 		ctx.AddFinalizerFn(f)
 	}
 
+	t.Run("TestCRDs", testCRDs)
+	t.Run("TestServerTLS", testServerTLS(t, ns))
+
 	// t.Run blocks until the function passed as the second argument (f) returns or
 	// calls t.Parallel to become a parallel test. Run reports whether f succeeded
 	// (or at least did not fail before calling t.Parallel). As all tests in
 	// testAllNS are parallel, the defered ctx.Cleanup above would be run before
-	// all tests finished. Wrapping it in testAllNS fixes this.
-	t.Run("x", testAllNS)
+	// all tests finished. Wrapping it in testAllNSPrometheus and testAllNSAlertmanager
+	// fixes this.
+	t.Run("x", testAllNSAlertmanager)
+	t.Run("y", testAllNSPrometheus)
+	t.Run("z", testAllNSThanosRuler)
 
 	// Check if Prometheus Operator ever restarted.
 	opts := metav1.ListOptions{LabelSelector: fields.SelectorFromSet(fields.Set(map[string]string{
 		"app.kubernetes.io/name": "prometheus-operator",
 	})).String()}
 
-	pl, err := framework.KubeClient.CoreV1().Pods(ns).List(opts)
+	pl, err := framework.KubeClient.CoreV1().Pods(ns).List(context.TODO(), opts)
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -99,8 +124,8 @@ func TestAllNS(t *testing.T) {
 	if err != nil {
 		t.Fatalf("failed to retrieve restart count of Prometheus Operator pod: %v", err)
 	}
-	if len(restarts) != 2 {
-		t.Fatalf("expected to have 2 containers but got %d", len(restarts))
+	if len(restarts) != 1 {
+		t.Fatalf("expected to have 1 container but got %d", len(restarts))
 	}
 	for _, restart := range restarts {
 		if restart != 0 {
@@ -112,9 +137,30 @@ func TestAllNS(t *testing.T) {
 	}
 }
 
-func testAllNS(t *testing.T) {
+func testCRDs(t *testing.T) {
+	const (
+		prometheusCRDName = "prometheuses.monitoring.coreos.com"
+	)
+	crds, err := framework.ListCRDs()
+	if err != nil {
+		t.Fatalf("unable to list CRDs: %v", err)
+	}
+	if len(crds.Items) < 5 {
+		t.Fatalf("incorrect number of CRDs, want at least: %v, got %v", 5, len(crds.Items))
+	}
+	crd, err := framework.GetCRD(prometheusCRDName)
+	if err != nil {
+		t.Fatalf("unable to get prometheus custom resource definition: %v", err)
+	}
+	// This field might be nil in older versions of kube (<1.15)
+	if crd.Spec.PreserveUnknownFields != nil && *crd.Spec.PreserveUnknownFields {
+		t.Fatalf("incorrect setting for preserveUnknownFields, want: %v, got: %v", false, *crd.Spec.PreserveUnknownFields)
+	}
+}
+
+func testAllNSAlertmanager(t *testing.T) {
+	skipAlertmanagerTests(t)
 	testFuncs := map[string]func(t *testing.T){
-		// Alertmanager
 		"AMCreateDeleteCluster":           testAMCreateDeleteCluster,
 		"AMScaling":                       testAMScaling,
 		"AMVersionMigration":              testAMVersionMigration,
@@ -125,13 +171,22 @@ func testAllNS(t *testing.T) {
 		"AMClusterGossipSilences":         testAMClusterGossipSilences,
 		"AMReloadConfig":                  testAMReloadConfig,
 		"AMZeroDowntimeRollingDeployment": testAMZeroDowntimeRollingDeployment,
+	}
 
-		// Prometheus
+	for name, f := range testFuncs {
+		t.Run(name, f)
+	}
+}
+
+func testAllNSPrometheus(t *testing.T) {
+	skipPrometheusTests(t)
+	testFuncs := map[string]func(t *testing.T){
 		"PromCreateDeleteCluster":                testPromCreateDeleteCluster,
 		"PromScaleUpDownCluster":                 testPromScaleUpDownCluster,
 		"PromNoServiceMonitorSelector":           testPromNoServiceMonitorSelector,
 		"PromVersionMigration":                   testPromVersionMigration,
 		"PromResourceUpdate":                     testPromResourceUpdate,
+		"PromStorageLabelsAnnotations":           testPromStorageLabelsAnnotations,
 		"PromStorageUpdate":                      testPromStorageUpdate,
 		"PromReloadConfig":                       testPromReloadConfig,
 		"PromAdditionalScrapeConfig":             testPromAdditionalScrapeConfig,
@@ -149,7 +204,9 @@ func testAllNS(t *testing.T) {
 		"PromExposingWithKubernetesAPI":          testPromExposingWithKubernetesAPI,
 		"PromDiscoverTargetPort":                 testPromDiscoverTargetPort,
 		"PromOpMatchPromAndServMonInDiffNSs":     testPromOpMatchPromAndServMonInDiffNSs,
-		"PromGetBasicAuthSecret":                 testPromGetBasicAuthSecret,
+		"PromGetAuthSecret":                      testPromGetAuthSecret,
+		"PromArbitraryFSAcc":                     testPromArbitraryFSAcc,
+		"PromTLSConfigViaSecret":                 testPromTLSConfigViaSecret,
 		"Thanos":                                 testThanos,
 	}
 
@@ -158,6 +215,16 @@ func testAllNS(t *testing.T) {
 	}
 }
 
+func testAllNSThanosRuler(t *testing.T) {
+	skipThanosRulerTests(t)
+	testFuncs := map[string]func(t *testing.T){
+		"ThanosRulerCreateDeleteCluster": testTRCreateDeleteCluster,
+	}
+	for name, f := range testFuncs {
+		t.Run(name, f)
+	}
+}
+
 // TestMultiNS tests the Prometheus Operator configured to watch specific
 // namespaces.
 func TestMultiNS(t *testing.T) {
@@ -172,6 +239,7 @@ func TestMultiNS(t *testing.T) {
 
 // TestDenylist tests the Prometheus Operator configured not to watch specific namespaces.
 func TestDenylist(t *testing.T) {
+	skipPrometheusTests(t)
 	testFuncs := map[string]func(t *testing.T){
 		"Prometheus":     testDenyPrometheus,
 		"ServiceMonitor": testDenyServiceMonitor,
@@ -181,3 +249,50 @@ func TestDenylist(t *testing.T) {
 		t.Run(name, f)
 	}
 }
+
+// TestPromInstanceNs tests prometheus operator in different scenarios when --prometheus-instance-namespace is given
+func TestPromInstanceNs(t *testing.T) {
+	skipPrometheusTests(t)
+	testFuncs := map[string]func(t *testing.T){
+		"AllNs":     testPrometheusInstanceNamespaces_AllNs,
+		"AllowList": testPrometheusInstanceNamespaces_AllowList,
+		"DenyList":  testPrometheusInstanceNamespaces_DenyList,
+	}
+
+	for name, f := range testFuncs {
+		t.Run(name, f)
+	}
+}
+
+// TestAlertmanagerInstanceNs tests prometheus operator in different scenarios when --alertmanager-instance-namespace is given
+func TestAlertmanagerInstanceNs(t *testing.T) {
+	skipAlertmanagerTests(t)
+	testFuncs := map[string]func(t *testing.T){
+		"AllNs":  testAlertmanagerInstanceNamespaces_AllNs,
+		"DenyNs": testAlertmanagerInstanceNamespaces_DenyNs,
+	}
+
+	for name, f := range testFuncs {
+		t.Run(name, f)
+	}
+}
+
+const (
+	prometheusOperatorServiceName = "prometheus-operator"
+)
+
+func testServerTLS(t *testing.T, namespace string) func(t *testing.T) {
+
+	return func(t *testing.T) {
+		if err := operatorFramework.WaitForServiceReady(framework.KubeClient, namespace, prometheusOperatorServiceName); err != nil {
+			t.Fatal("waiting for prometheus operator service: ", err)
+		}
+
+		operatorService := framework.KubeClient.CoreV1().Services(namespace)
+		request := operatorService.ProxyGet("https", prometheusOperatorServiceName, "https", "/healthz", make(map[string]string))
+		_, err := request.DoRaw(context.TODO())
+		if err != nil {
+			t.Fatal(err)
+		}
+	}
+}
